## This function is used by mjplot.

## Mats Jansson, May 9 2000 - June 7 2000
## Report bugs to: mats@nera.no

function [status, set_style, indices, val] = __mjpltopt__ (caller, opt, arg)
  
  fname = "__mjpltopt__";
  if (nargin < 2 || nargin > 3),
    error ("%s: expecting two or three arguments", fname);
  elseif (! isstr (opt)),
    error ("%s: second argument should be a string", fname);
  endif
  
  status = 0; ## initial value
  ## status:
  ## 0: opt is a line specification (eg. "-@r;s21;") or opt is a switch
  ##    which have a default value and __mjpltopt__ only got two
  ##    arguments.
  ## 1: at least one row of opt is a switch and __mjpltopt__ needs
  ##    three arguments (but only got two)
  ## 2: at least one row of opt is a switch and __mjpltopt__ got three
  ##    arguments.

  switches = ["Axes           ";
              "Color          ";
              "LineStyle      ";
              "LineType       ";
              "LineWidth      ";
              "LS             ";
              "LT             ";
              "LW             ";
              "Marker         ";
              "MarkerEdgeColor";
              "MarkerFaceColor";
              "MarkerSize     ";
              "PointSize      ";
              "PointType      ";
              "PS             ";
              "PT             ";
              "Smooth         ";
              "Style          ";
              "Title          ";
              "Using          "];
  if (isempty (opt)),
    opt = " ";
  endif
  [nor, noc] = size (opt);
  [nsr, nsc] = size (switches);
  isswitch = zeros (nor, 1);
  if (noc < nsc),
    ## Make room for the longest switch (MarkerEdgeColor) in the string-
    ## matrix opt.
    opt = [opt, setstr (toascii (" ") * ones (nor, (nsc - noc)))];
  endif
  for n = 1:nor
    tmp = abbrev (deblank (opt(n,:)), switches);
    if (! isempty (tmp)),
      isswitch(n) = 1;
      opt(n,1:length (tmp)) = tmp;
    endif
  endfor
  if (any (isswitch) && nargin == 3),
    ## At least one row of opt is a switch and we have three arguments. 
    status = 2;
    [nar, nac] = size (arg);
    ## opt and arg should have the same number of rows, repeat the last
    ## row of the shortest one
    if (nar == 0 || nac == 0),
      arg = setstr (toascii (" ") * ones (nor, 1));
    elseif (nor > nar),
      arg = [arg; repmat (arg(nar,:), nor - nar, 1)];
    elseif (nor < nar),
      opt = [opt; repmat (opt(nor,:), nar - nor, 1)];
      isswitch = [isswitch; isswitch(nor) * ones (nar - nor, 1)];
      nor = nar;
    endif
  elseif (any (isswitch)),
    arg = setstr (toascii (" ") * ones (nor, 1));
  endif

  set_style = zeros (nor, 18); ## initial value
  ## set_style:
  ## column 1: lines
  ##        2: points
  ##        3: linespoints
  ##        4: impulses
  ##        5: dots
  ##        6: steps
  ##        7: fsteps
  ##        8: histeps
  ##        9: errorbars
  ##       10: xerrorbars
  ##       11: yerrorbars
  ##       12: xyerrorbars
  ##       13: boxes
  ##       14: boxerrorbars
  ##       15: boxxyerrorbars
  ##       16: financebars
  ##       17: candlesticks
  ##       18: vector
  
  indices = zeros (nor, 16); ## initial value
  ## indices:
  ## column 1: linetype startindex in val
  ##        2: linetype stopindex in val
  ##        3: linewidth startindex
  ##        4: linewidth stopindex
  ##        5: pointtype startindex
  ##        6: pointtype stopindex
  ##        7: pointsize startindex
  ##        8: pointsize stopindex
  ##        9: key startindex
  ##       10: key stopindex
  ##       11: smooth startindex
  ##       12: smooth stopindex  
  ##       13: using startindex
  ##       14: using stopindex
  ##       15: axes startindex
  ##       16: axes stopindex

  val = ""; ## initial value
  val_ind = 0;
  
  for  n = 1:nor
    tl = deblank (opt(n,:));
    if (isempty (tl)),
      ## Do nothing
    elseif (isswitch(n)),
      if (strcmp (tl, "Style") ||
          strcmp (tl, "LineStyle") ||
          strcmp (tl, "LS")),
        set_style(n,__mjpltoptstyle__ (caller, arg(n,:))) = 1;
      elseif (strcmp (tl, "LineWidth") || strcmp (tl, "LW")),
        if (nargin == 2),
          ## The switch "LineWidth doesn't have a default value
          status = 1;
          return
        endif
        tmp = __mjpltoptlw__ (caller, arg(n,:));
        if (! isempty (tmp)),
          indices(n,3) = val_ind + 1;
          indices(n,4) = val_ind = val_ind + length (tmp);
          val = sprintf ("%s%s", val, tmp);
        endif
      elseif (strcmp (tl, "PointType") || \
              strcmp (tl, "PT")        || \
              strcmp (tl, "Marker")),
        if (nargin == 2),
          ## The switch "PointType" doesn't have a default value
          status = 1;
          return
        endif
        tmp = __mjpltoptpt__ (caller, arg(n,:));
        if (! isempty (tmp)),
          set_style(n,2) = 1;
          indices(n,5) = val_ind + 1;
          indices(n,6) = val_ind = val_ind + length (tmp);
          val = sprintf ("%s%s", val, tmp);
        endif
      elseif (strcmp (tl, "PointSize") || \
              strcmp (tl, "PS") || \
              strcmp (tl, "MarkerSize")),
        if (nargin == 2),
          ## The switch "PointSize" doesn't have a default value
          status = 1;
          return
        endif
        tmp = __mjpltoptlw__ (caller, arg(n,:));
        if (! isempty (tmp)),
          set_style(n,2) = 1;
          indices(n,7) = val_ind + 1;
          indices(n,8) = val_ind = val_ind + length (tmp);
          val = sprintf ("%s%s", val, tmp);
        endif
      elseif (strcmp (tl, "Title")),
        ## The switch "Title" doesn't have a default value
        if nargin == 2
          status = 1;
          return
        endif
        if (! isstr (arg)),
          error ("%s: argument for Title should be a string", caller);
        endif
        tmp = deblank (arg(n,:));
        indices(n,9) = val_ind + 1;
        indices(n,10) = val_ind = val_ind + length (tmp);
        val = sprintf ("%s%s", val, tmp);
      elseif (strcmp (tl, "LineType")        || \
              strcmp (tl, "LT")              || \
              strcmp (tl, "Color")           || \
              strcmp (tl, "MarkerEdgeColor") || \
              strcmp (tl, "MarkerFaceColor")),
        tmp = __mjpltoptlt__ (caller, arg(n,:));
        if (! isempty (tmp)),
          indices(n,1) = val_ind + 1;
          indices(n,2) = val_ind = val_ind + length (tmp);
          val = sprintf ("%s%s", val, tmp);
        endif
      elseif (strcmp (tl, "Smooth")),
        if (nargin == 2),
          tmp = __mjpltoptsmooth__ (caller, "");
        else
          tmp = __mjpltoptsmooth__ (caller, arg(n,:));
        endif
        indices(n,11) = val_ind + 1;
        indices(n,12) = val_ind = val_ind + length (tmp);
        val = sprintf ("%s%s", val, tmp);
      elseif (strcmp (tl, "Using")),
        if (nargin == 2),
          status = 1;
          return
        endif
        tmp = __mjpltoptusing__ (caller, arg(n,:));
        if (! isempty (tmp)),
          indices(n,13) = val_ind + 1;
          indices(n,14) = val_ind = val_ind + length (tmp);
          val = sprintf ("%s%s", val, tmp);
        endif
      elseif (strcmp (tl, "Axes")),
        if (nargin == 2),
          status = 1;
          return
        endif
        tmp = __mjpltoptaxes__ (caller, arg(n,:));
        if (! isempty (tmp)),
          indices(n,15) = val_ind + 1;
          indices(n,16) = val_ind = val_ind + length (tmp);
          val = sprintf ("%s%s", val, tmp);
        endif
      else
        error ("%s: Somthing went wrong here...", fname);
      endif
    else
      ## tl is not a switch; test it for line specifications
      if (! isempty (sc_index = find (tl == ";"))),
        ## Set the title and remove the title including the semicolons
        ## from the string 'tl'.
        if (prod (size (sc_index)) < 2),
          error ("%s: unfinished key label", caller);
        endif
        tmp = tl(min (sc_index) + 1:max (sc_index - 1));
        indices(n,9) = val_ind + 1;
        indices(n,10) = val_ind = val_ind + length (tmp);
        val = sprintf ("%s%s", val, tmp);
        tl = sprintf ("%s%s", \
                      tl(1:min (sc_index) - 1), \
                      tl(max (sc_index) + 1:length (tl)));
      endif
      if (! isempty (tl)),
        if (any (used_ind = (tl == "-" | \ 
                             tl == "." | \
                             tl == ":" | \
                             tl == "@" | \
                             tl == "i" | \
                             tl == "L" | \
                             tl == "#" | \
                             tl == "~"))),
          ## Set the style
          if (any (tl == "-")),
            set_style(n,1) = 1;
          endif
          if (any (tl == "@")),
            set_style(n,2) = 1;
          endif
          if (any (tl == "i")),
            set_style(n,4) = 1;
          endif
          if (any (tl == "." | tl == ":")),
            set_style(n,5)  = 1;
          endif
          if (any (tl == "L")),
            set_style(n,6) = 1;
          endif
          if (any (tl == "~")),
            set_style(n,9) = 1;
          endif
          if (any (tl == "#")),
            set_style(n,13) = 1;
          endif
        endif
        
        if (any (tmp = (tl == "r" | \ ## line-type 1
                        tl == "g" | \ ## line-type 2
                        tl == "b" | \ ## line-type 3
                        tl == "m" | \ ## line-type 4
                        tl == "c" | \ ## line-type 5
                        tl == "w" | \ ## line-type 6
                        tl == "k" | \ ## line-type 6
                        tl == "y"))), ## line-type 6
          used_ind = used_ind | tmp;
          tmp = find (tmp);
          if (length (tmp) > 1),
            ## Don't want to use  the warning function here because this
            ## function is usually called from within a try-catch block
            ## and hence the warnings will not be shown.
            disp (sprintf ("warning: %s: line-type are specified more than ones for the same plot (using the last one)", caller));
          endif
          char = tl(tmp(length(tmp))); ## If more than one line-type
                                       ## was given, use the last one.
          val = sprintf ("%s%d", val, min (6, find (char == "rgbmcwky")));
          indices(n,1) = indices(n,2) = ++val_ind;
        endif
        
        if (any (tmp = (tl == "o" | \ ## point-type 1
                        tl == "d" | \ ## point-type 1
                        tl == "+" | \ ## point-type 2
                        tl == "s" | \ ## point-type 3
                        tl == "x" | \ ## point-type 4
                        tl == "^" | \ ## point-type 5
                        tl == "*" | \ ## point-type 6
                        tl == "<" | \ ## point-type 6
                        tl == ">" | \ ## point-type 6
                        tl == "h" | \ ## point-type 6
                        tl == "p" | \ ## point-type 6
                        tl == "v"))), ## point-type 6
          
          used_ind = used_ind | tmp;
          tmp = find (tmp);
          if (length (tmp) > 1),
            disp (sprintf ("warning: %s: point-type are specified more than ones for the same plot (using the last one)", caller));
          endif
          char = tl(tmp(length(tmp)));
          number = [1, 1, 2, 3, 4, 5, 6, 6, 6, 6, 6, 6];
          val = sprintf ("%s%d", val, number(find (char == \
                                                   "od+sx^*<>hpv")));
          indices(n,5) = indices(n,6) = ++val_ind;
          set_style(n,2) = 1;
        endif
        
        if (any (tmp = (tl == "0" | \
                        tl == "1" | \
                        tl == "2" | \
                        tl == "3" | \
                        tl == "4" | \
                        tl == "5" | \
                        tl == "6" | \
                        tl == "7" | \
                        tl == "8" | \
                        tl == "9"))),
          used_ind = used_ind | tmp;
          tmp = find (tmp);
          siz = length (tmp);
          if (! indices(n,1)),
            ## Set the line-type
            val = sprintf ("%s%s", val, tl(tmp(1)));
            indices(n,1) = indices(n,2) = ++val_ind;
            if (siz >= 2 && indices(n,5)),
              disp (sprintf ("warning: %s: the point-type is given as a string argument (ignoring numeric argument(s))", caller));
            elseif (siz >= 2 && ! indices(n,5)),
              ## Set the point-type
              val = sprintf ("%s%s", val, tl(tmp(2)));
              indices(n,5) = indices(n,6) = ++val_ind;
              set_style(n,2) = 1; ## Set point style
              if (siz > 2),
                disp (sprintf ("warning: %s: more than two numeric line-specifications are given", caller));
              endif
            endif
          elseif (! indices(n,5)),
            ## Set only point-type
            val = sprintf ("%s%s", val, tl(tmp(1)));
            indices(n,5) = indices(n,6) = ++val_ind;
            set_style(n,2) = 1; ## Set point style
            if (siz > 1),
              disp (sprintf ("warning: %s: using the first numeric argument as point-type (ignoring additional numeric argument(s))", caller));
            endif
          else
            disp (sprintf ("warning: %s: line- and point-type are given as string arguments (ignoring numeric argument(s))", caller));
          endif
        endif
        
        if (any (tmp = (tl == " "))), 
          ## ignore spaces
          used_ind = used_ind | tmp;
        endif
        
        if (! isempty (tmp = find (used_ind == 0))),
          error ("%s: unrecognized format character: '%s' or unrecognized switch: '%s'", caller, tl(tmp(1)), tl);
        endif
      endif
    endif
  endfor
endfunction
