## function retval = abbrev (s, l)
##
## If the string row vector s is a unique abbreviation of one row in the
## string matrix l or if s is identical to one row in l; return that row
## (after deblank). Otherwise return the empty string.

## The algorithm:
##
## Assume s is "bet" and l is ["alpha"; "beta"; "gamma"].
##
## 1) Make a matrix of 's', with as many rows as in 'l'. Each row of the
##    matrix s is equal.
##         _   _
##        | bet |
##    s = | bet |
##        |_bet_|
##
## 2) Transpose s and l.
##                       _   _
##         _   _        | abg |
##        | bbb |       | lea |
##    s = | eee |   l = | ptm |
##        |_ttt_|       | ham |
##                      |_a a_|
##
## 3) Check if 's' is an abbreviation by comparing the first rows (three
##    rows in this expample).
##
##    all (s == l(1:3,:)) = [0, 1, 0]
## 
##    In this example, we can see that 's' is a unique abbreviation of
##    "beta".
##
##    If we get only zeros in the comparison, there is no match. If we
##    get more than one ones in the comparison, the abbreviation is not
##    unique. In this case we have to check if 's' is a perfect match of
##    one string in 'l'.

## Mats Jansson, May 10, 2000
## Report bugs to: mats@nera.no

function retval = abbrev (s, l)

  if (nargin == 2),
    [r_s, c_s] = size (s);
    
    if ((! isstr (s)) || (! isstr (l)) || (r_s > 1)),
      error ("abbrev: expecting two string arguments (one row vector and one matrix)");
    endif

    l = deblank (l);
    [r_l, c_l] = size (l);
    if (c_s > c_l || isempty (s)),
      ## s is longer than l and can not be a abbreviation of any row in
      ## l or s is empty.
      retval = "";
    else
      s = (s(ones (r_l, 1), (1:c_s)'))'; ## Make a matrix of s and
                                         ## transpose.
      l = l';
      if (c_s == 1),
        hits = find (s == l(1,:)); ##  Cant use 'all' when comparing
                                   ##  just one row.
      else
        hits = find (all (s == l(1:c_s,:)));
      endif
      if (isempty (hits)),
        ## No match
        retval = "";
      elseif (max (size (hits)) == 1 || c_s == c_l),
        retval =  deblank ((l(:,hits(1)))');
      else
        ## more than one match; check if s is a full name
        s = [s; setstr (ones (c_l - c_s, r_l) * toascii (" "))];
        hits = find (all (s == l));
        if (isempty (hits)),
          ## s is not an unique abbreviation
          retval = "";
        else
          retval =  deblank ((l(:,hits(1)))');
        endif
      endif
    endif
  else
    error ("abbrev: expecting two string arguments (one row vector and one matrix)");
  endif
endfunction
